<?php
namespace Aheadworks\Followupemail2\Helper;

use Aheadworks\Followupemail2\Api\Data\EventHistoryInterface;
use Aheadworks\Followupemail2\Api\Data\EventInterface;
use Magento\Customer\Api\Data\CustomerInterface;
use Magento\Customer\Api\Data\GroupInterface;
use Magento\Framework\Exception\NoSuchEntityException;
use Magento\Newsletter\Model\Subscriber as NewsletterSubscriber;
use Magento\Quote\Api\CartRepositoryInterface;
use Aheadworks\Followupemail2\Api\EventHistoryManagementInterface;
use Aheadworks\Followupemail2\Api\EventHistoryRepositoryInterface;
use Magento\Customer\Model\Customer;
use Magento\Customer\Api\GroupManagementInterface;
use Magento\Customer\Api\CustomerRepositoryInterface;
use Magento\Sales\Api\Data\OrderInterface;
use Magento\Sales\Model\ResourceModel\Order as OrderResource;
use Magento\Sales\Model\Order;
use Aheadworks\Followupemail2\Model\Serializer;
/**
 * Class Data
 * @package Aheadworks\Followupemail2\Helper
 */
class Data
{
    /**
     * @var CartRepositoryInterface
     */
    private $quoteRepository;
    /**
     * @var EventHistoryManagementInterface
     */
    private $eventHistoryManagement;
    /**
     * @var EventHistoryRepositoryInterface
     */
    private $eventHistoryRepository;
    /**
     * @var Customer
     */
    private $customer;
    /**
     * @var NewsletterSubscriber
     */
    private $subscriber;
    /**
     * @var GroupManagementInterface
     */
    private $groupManagement;
    /**
     * @var CustomerRepositoryInterface
     */
    private $customerRepository;
    /**
     * @var Order
     */
    private $order;
    /**
     * @var Serializer
     */
    private $serializer;


    /**
     * Data constructor.
     * @param CartRepositoryInterface $quoteRepository
     * @param EventHistoryManagementInterface $eventHistoryManagement
     * @param EventHistoryRepositoryInterface $eventHistoryRepository
     * @param Customer $customer
     * @param NewsletterSubscriber $subscriber
     * @param GroupManagementInterface $groupManagement
     * @param CustomerRepositoryInterface $customerRepository
     * @param Order $order
     * @param Serializer $serializer
     */
    public function __construct(
        CartRepositoryInterface $quoteRepository,
        EventHistoryManagementInterface $eventHistoryManagement,
        EventHistoryRepositoryInterface $eventHistoryRepository,
        Customer $customer,
        NewsletterSubscriber $subscriber,
        GroupManagementInterface $groupManagement,
        CustomerRepositoryInterface  $customerRepository,
        Order $order,
        Serializer $serializer){
        $this->quoteRepository = $quoteRepository;
        $this->eventHistoryManagement = $eventHistoryManagement;
        $this->eventHistoryRepository = $eventHistoryRepository;
        $this->customer = $customer;
        $this->subscriber = $subscriber;
        $this->groupManagement = $groupManagement;
        $this->customerRepository = $customerRepository;
        $this->order = $order;
        $this->serializer = $serializer;
    }


    /**
     * @param EventHistoryInterface $eventHistoryItem
     * @return bool
     * @throws \Magento\Framework\Exception\NoSuchEntityException
     */
    public function isRevoverableCart(EventHistoryInterface $eventHistoryItem){
        try{
            $cartId = $eventHistoryItem->getReferenceId();
            $quote = $this->quoteRepository->get($cartId);
            if(!$quote->getCustomerIsGuest()){
                $customerEmail = $quote->getCustomer()->getEmail();
                $customer=$quote->getCustomer();
                if($customer->getFirstname() && $customer->getLastname()){
                    $customerName = $customer->getFirstname() . ' '. $customer->getLastname();
                }
                else if(count($customer->getAddresses())){
                    $customerName = $quote->getBillingAddress()->getFirstname() . ' ' . $quote->getBillingAddress()->getLastname();
                }else{
                    $eventHistoryItem->setProcessed(1);
                    $this->eventHistoryRepository->save($eventHistoryItem);
                    return false;
                }
                $cartData = array_merge($quote->getData(), [
                    'email' => $customerEmail,
                    'customer_name' => $customerName
                ]);
                $eventHistoryItem->setEventData($this->getPreparedEventData($cartData));
                $this->eventHistoryRepository->save($eventHistoryItem);
                return true;
            }else{
                //delete data
                $eventHistoryItem->setProcessed(1);
                $this->eventHistoryRepository->save($eventHistoryItem);
                return false;
            }
        }catch (\Exception $e){
            //delete data
            $eventHistoryItem->setProcessed(1);
            $this->eventHistoryRepository->save($eventHistoryItem);
            return false;
        }catch (\Throwable $t){
            //delete data
            $eventHistoryItem->setProcessed(1);
            $this->eventHistoryRepository->save($eventHistoryItem);
            return false;
        }

    }

    /**
     * @param EventHistoryInterface $eventHistoryItem
     * @return bool
     * @throws \Magento\Framework\Exception\NoSuchEntityException
     */
    public function isRecoverableCustomer(EventHistoryInterface $eventHistoryItem){
        try {

            $customer = $this->customer->load($eventHistoryItem->getReferenceId());
            if ($customer) {
                $customerData = array_merge(
                    $customer->getData(),
                    [
                        'customer_name' => $customer->getName(),
                        'customer_group_id' => $customer->getGroupId()
                    ]
                );
                if(isset($customerData['password_hash'])){
                    unset($customerData['password_hash']);
                }
                $eventHistoryItem->setEventData($this->getPreparedEventData($customerData));
                $this->eventHistoryRepository->save($eventHistoryItem);
                return true;
            }else{
                $eventHistoryItem->setProcessed(1);
                $this->eventHistoryRepository->save($eventHistoryItem);
                return false;
            }

        }catch (\Exception $e){
            $eventHistoryItem->setProcessed(1);
            $this->eventHistoryRepository->save($eventHistoryItem);
            return false;
        }catch (\Throwable $t){
            $eventHistoryItem->setProcessed(1);
            $this->eventHistoryRepository->save($eventHistoryItem);
            return false;
        }
    }

    /**
     * @param EventHistoryInterface $eventHistoryItem
     * @return bool
     * @throws \Magento\Framework\Exception\LocalizedException
     */
    public function isRecoverableNewsletter(EventHistoryInterface $eventHistoryItem){
        try{
            $customerName = '';
            $subscriber = $this->subscriber->load($eventHistoryItem->getReferenceId());
            if(!$subscriber){
                $eventHistoryItem->setProcessed(1);
                $this->eventHistoryRepository->save($eventHistoryItem);
                return false;
            }
            /** @var GroupInterface $notLoggedInGroup */
            $notLoggedInGroup = $this->groupManagement->getNotLoggedInGroup();
            $customerGroupId = $notLoggedInGroup->getId();
            if ($subscriber->getCustomerId()) {
                try {
                    /** @var CustomerInterface $customer */
                    $customer = $this->customerRepository->getById($subscriber->getCustomerId());
                    $customerName = $customer->getFirstname() . ' ' . $customer->getLastname();
                    $customerGroupId = $customer->getGroupId();
                } catch (NoSuchEntityException $e) {
                }
            }

            $subscriberData = array_merge($subscriber->getData(), [
                'email'             => $subscriber->getSubscriberEmail(),
                'customer_name'     => $customerName,
                'customer_group_id' => $customerGroupId

            ]);

            $eventHistoryItem->setEventData($this->getPreparedEventData($subscriberData));
            $this->eventHistoryRepository->save($eventHistoryItem);
            return true;
        }catch (\Exception $e){
            $eventHistoryItem->setProcessed(1);
            $this->eventHistoryRepository->save($eventHistoryItem);
            return false;
        }catch (\Throwable $t){
            $eventHistoryItem->setProcessed(1);
            $this->eventHistoryRepository->save($eventHistoryItem);
            return false;
        }
    }

    /**
     * @param EventHistoryInterface $eventHistoryItem
     * @return bool
     * @throws \Magento\Framework\Exception\LocalizedException
     */
    public function isRecoverableOrderStatusChanged(EventHistoryInterface $eventHistoryItem){
        try{
            $order = $this->order->load($eventHistoryItem->getReferenceId());
            if(!$order){
                $eventHistoryItem->setProcessed(1);
                $this->eventHistoryRepository->save($eventHistoryItem);
                return false;
            }
            $orderIdBeforeSave = $order->getId();

            if ($order->getCustomerId()) {
                $customerFirstName = $order->getCustomerFirstname();
                $customerName = $order->getCustomerFirstname() . ' ' . $order->getCustomerLastname();
            } else {
                $customerFirstName = $order->getBillingAddress()->getFirstname();
                $customerName = $order->getBillingAddress()->getName();
            }
            $orderData = array_merge(
                $order->getData(),
                [
                    'email' => $order->getCustomerEmail(),
                    'customer_firstname' => $customerFirstName,
                    'customer_name' => $customerName
                ]
            );
            $eventHistoryItem->setEventData($this->getPreparedEventData($orderData));
            $this->eventHistoryRepository->save($eventHistoryItem);
            return true;
        }catch (\Exception $e){
            $eventHistoryItem->setProcessed(1);
            $this->eventHistoryRepository->save($eventHistoryItem);
            return false;
        }catch (\Throwable $t){
            $eventHistoryItem->setProcessed(1);
            $this->eventHistoryRepository->save($eventHistoryItem);
            return false;
        }
    }

    /**
     * Get prepared event data
     *
     * @param array $data
     * @return string
     */
    private function getPreparedEventData(array $data)
    {
        foreach ($data as $key => $value) {
            if ((is_array($value) || is_object($value))) {
                unset($data[$key]);
            }

            if (isset($data[$key]) && preg_match("/\r\n|\r|\n/", $value)) {
                $data[$key] = preg_replace("/\r\n|\r|\n/", "", $value);
            }
        }
        return $this->serializer->serialize($data);
    }

}